﻿using System;
using System.Collections.Generic;
using System.Linq;
using ProtoBuf;
using Utility;
using NLog;
using NLog.Config;
using NLog.Targets;
using System.Text;

namespace GameLib
{
    public static class NLogExtension
    {
        static Dictionary<string, FileTarget> mapModules = new Dictionary<string, FileTarget>();

        public static void AddModuleFile(this LoggingConfiguration config, string moduleName, string loggerNamePattern)
        {
            loggerNamePattern = "*" + loggerNamePattern + "*";
            if(!mapModules.ContainsKey(moduleName))
            {
                // 添加文件
                FileTarget fileTarget = new FileTarget();
                fileTarget.FileName = "${basedir}/Log/${shortdate}/" + moduleName + ".log";
                fileTarget.BufferSize = 6 * 1024 * 1024;
                fileTarget.Encoding = Encoding.UTF8;
                fileTarget.KeepFileOpen = true;
                fileTarget.AutoFlush = false;
                fileTarget.Layout = "${longdate} ${level} ${message}";
                config.AddTarget(moduleName, fileTarget);

                LoggingRule rule2 = new LoggingRule(loggerNamePattern, LogLevel.Trace, fileTarget);
                config.LoggingRules.Add(rule2);

                mapModules.Add(moduleName, fileTarget);
            }
            else
            {
                LoggingRule rule2 = new LoggingRule(loggerNamePattern, LogLevel.Trace, mapModules[moduleName]);
                config.LoggingRules.Add(rule2);
            }

            LogManager.Configuration = config;
        }

        public static void ConfigureNLog(Config.LogConfig configIn)
        {
            LoggingConfiguration config = new LoggingConfiguration();
            
            // 添加控制台
            ColoredConsoleTarget consoleTarget = new ColoredConsoleTarget();
            consoleTarget.Layout = "${longdate} ${level} ${message}";
            config.AddTarget("console", consoleTarget);

            string processname = SysHelp.GetProcessName();

            // 添加文件
            FileTarget fileTarget = new FileTarget();
            fileTarget.FileName = "${basedir}/Log/${shortdate}/" + processname + ".log";
            fileTarget.BufferSize = 6 * 1024 * 1024;
            fileTarget.Encoding = Encoding.UTF8;
            fileTarget.KeepFileOpen = true;
            fileTarget.AutoFlush = false;
            fileTarget.Layout = "${longdate} ${level} ${message}";
            config.AddTarget("file", fileTarget);

            // 添加warn文件
            FileTarget fileTargetWarn = new FileTarget();
            fileTargetWarn.FileName = "${basedir}/Log/${shortdate}/" + processname + "_Warn.log";
            fileTargetWarn.BufferSize = 6 * 1024 * 1024;
            fileTargetWarn.Encoding = Encoding.UTF8;
            fileTargetWarn.KeepFileOpen = true;
            fileTargetWarn.AutoFlush = false;
            fileTargetWarn.Layout = "${longdate} ${level} ${message}";
            config.AddTarget("file_warn", fileTargetWarn);

            if (configIn.IsConsoleScreen)
            {
                LoggingRule rule1 = new LoggingRule("*", LogLevel.Trace, consoleTarget);
                config.LoggingRules.Add(rule1);
            }
            else
            {
                LoggingRule rule1 = new LoggingRule("*", LogLevel.Warn, consoleTarget);
                config.LoggingRules.Add(rule1);
            }

            LoggingRule rule2 = new LoggingRule("*", LogLevel.Trace, fileTarget);
            config.LoggingRules.Add(rule2);

            LoggingRule rule3 = new LoggingRule("*", LogLevel.Warn, fileTargetWarn);
            config.LoggingRules.Add(rule3);

            LogManager.Configuration = config;
        }
    };
}
